from pymemri.data.schema import Item, Edge

class Account(Item):
    
    def __init__(self, service=None, identifier=None, secret=None, code=None, refreshToken=None, errorMessage=None, id=None, deleted=None): #e.g. username, password or access key, secret key etc.
        self.id = id
        self.deleted = deleted
        self.service = service
        self.identifier = identifier
        self.secret = secret
        self.refreshToken = refreshToken
        self.code = code
        self.errorMessage = errorMessage
        
    @classmethod
    def from_json(cls, json):
        id = json.get("id", None)
        deleted = json.get("deleted", None)
        service = json.get("service", None)
        identifier = json.get("identifier", None)
        secret = json.get("secret", None)
        code = json.get("code", None)
        refreshToken = json.get("refreshToken", None)
        errorMessage = json.get("errorMessage", None)

        res = cls(service=service, identifier=identifier, secret=secret, code=code, refreshToken=refreshToken, errorMessage=errorMessage, id=id, deleted=deleted)
        return res


class Plugin(Item):
    
    def __init__(self, name, container, account=None, id=None, deleted=None):
        super().__init__(id=id, deleted=deleted)
        self.name = name
        self.container = container
        self.account = account if account else []

    @classmethod
    def from_json(cls, json):
        id = json.get("id", None)
        deleted = json.get("deleted", None)
        name = json.get("name", None)
        container = json.get("container", None)
        all_edges = json.get("allEdges", None)
        
        account = []
        
        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "account" or edge._type == "~account": 
                    account.append(edge)
                
        res = cls(name=name, container=container, account=account, id=id, deleted=deleted)
        for e in res.get_all_edges(): e.source = res
        return res

class StartPlugin(Item):
    
    def __init__(self, targetItemId, container, state=None, oAuthUrl=None, interval=None, view=None, id=None, deleted=None):
        super().__init__(id=id, deleted=deleted)
        self.targetItemId = targetItemId
        self.container = container
        self.state = state
        self.oAuthUrl = oAuthUrl
        self.interval = interval
        # CVUStoredDefinitions
        self.view = view if view else []


    @classmethod
    def from_json(cls, json):
        id = json.get("id", None)
        deleted = json.get("deleted", None)
        targetItemId = json.get("targetItemId", None)
        container = json.get("container", None)
        state = json.get("state", None)
        oAuthUrl = json.get("oAuthUrl", None)
        interval = json.get("interval", None)
        all_edges = json.get("allEdges", None)

        view = []

        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "view" or edge._type == "~view": 
                    view.append(edge)

        res = cls(targetItemId=targetItemId, container=container, state=state, view=view, oAuthUrl=oAuthUrl, interval=interval, id=id, deleted=deleted)
        return res


class File(Item):
    def __init__(self, dateAccessed=None, dateCreated=None, dateModified=None, deleted=None,
                 externalId=None, itemDescription=None, starred=None, version=None, id=None, importJson=None,
                 sha256=None, nonce=None, key=None, filename=None, fileLocation=None, changelog=None, label=None,
                 genericAttribute=None, measure=None, sharedWith=None, resource=None, usedBy=None, folder=None):
        super().__init__(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                         deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                         version=version, id=id, importJson=importJson, changelog=changelog, label=label,
                         genericAttribute=genericAttribute, measure=measure, sharedWith=sharedWith)
        self.sha256 = sha256
        self.nonce = nonce
        self.key = key
        self.filename = filename
        self.fileLocation = fileLocation
        self.resource = resource if resource is not None else []
        self.usedBy = usedBy if usedBy is not None else []
        self.folder = folder if folder is not None else []

    @classmethod
    def from_json(cls, json):
        all_edges = json.get("allEdges", None)
        dateAccessed = json.get("dateAccessed", None)
        dateCreated = json.get("dateCreated", None)
        dateModified = json.get("dateModified", None)
        deleted = json.get("deleted", None)
        externalId = json.get("externalId", None)
        itemDescription = json.get("itemDescription", None)
        starred = json.get("starred", None)
        version = json.get("version", None)
        id = json.get("id", None)
        importJson = json.get("importJson", None)
        sha256 = json.get("sha256", None)
        nonce = json.get("nonce", None)
        key = json.get("key", None)
        filename = json.get("filename", None)
        fileLocation = json.get("fileLocation", None)


        changelog = []
        label = []
        genericAttribute = []
        measure = []
        sharedWith = []
        resource = []
        usedBy = []
        folder = []

        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "changelog" or edge._type == "~changelog":
                    changelog.append(edge)
                elif edge._type == "label" or edge._type == "~label":
                    label.append(edge)
                elif edge._type == "genericAttribute" or edge._type == "~genericAttribute":
                    genericAttribute.append(edge)
                elif edge._type == "measure" or edge._type == "~measure":
                    measure.append(edge)
                elif edge._type == "sharedWith" or edge._type == "~sharedWith":
                    sharedWith.append(edge)
                elif edge._type == "resource" or edge._type == "~resource":
                    resource.append(edge)
                elif edge._type == "usedBy" or edge._type == "~usedBy":
                    usedBy.append(edge)
                elif edge._type == "folder" or edge._type == "~folder":
                    folder.append(edge)

        res = cls(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                  deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                  version=version, id=id, importJson=importJson, sha256=sha256, nonce=nonce, key=key,
                  filename=filename, fileLocation=fileLocation, changelog=changelog, label=label, genericAttribute=genericAttribute,
                  measure=measure, sharedWith=sharedWith, resource=resource, usedBy=usedBy, folder=folder)
        for e in res.get_all_edges(): e.source = res
        return res


class Photo(Item):
    def __init__(self, dateAccessed=None, dateCreated=None, dateModified=None, deleted=None,
                 externalId=None, itemDescription=None, starred=None, version=None, id=None, importJson=None,
                 bitrate=None, duration=None, endTime=None, fileLocation=None, startTime=None, caption=None,
                 exifData=None, name=None, height=None, width=None, channels=None, changelog=None, label=None,
                 genericAttribute=None, measure=None, sharedWith=None, file=None, includes=None, thumbnail=None,
                 folder=None,):
        super().__init__(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                         deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                         version=version, id=id, importJson=importJson, changelog=changelog, label=label,
                         genericAttribute=genericAttribute, measure=measure, sharedWith=sharedWith)
        self.bitrate = bitrate
        self.duration = duration
        self.endTime = endTime
        self.fileLocation = fileLocation
        self.startTime = startTime
        self.caption = caption
        self.exifData = exifData
        self.name = name
        self.height = height
        self.width = width
        self.channels = channels
        self.file = file if file is not None else []
        self.includes = includes if includes is not None else []
        self.thumbnail = thumbnail if thumbnail is not None else []
        self.folder = folder if folder is not None else []

    @classmethod
    def from_json(cls, json):
        all_edges = json.get("allEdges", None)
        dateAccessed = json.get("dateAccessed", None)
        dateCreated = json.get("dateCreated", None)
        dateModified = json.get("dateModified", None)
        deleted = json.get("deleted", None)
        externalId = json.get("externalId", None)
        itemDescription = json.get("itemDescription", None)
        starred = json.get("starred", None)
        version = json.get("version", None)
        id = json.get("id", None)
        importJson = json.get("importJson", None)
        bitrate = json.get("bitrate", None)
        duration = json.get("duration", None)
        endTime = json.get("endTime", None)
        fileLocation = json.get("fileLocation", None)
        startTime = json.get("startTime", None)
        caption = json.get("caption", None)
        exifData = json.get("exifData", None)
        name = json.get("name", None)
        height = json.get("height", None)
        width = json.get("width", None)
        channels = json.get("channels", None)

        changelog = []
        label = []
        genericAttribute = []
        measure = []
        sharedWith = []
        file = []
        includes = []
        thumbnail = []
        folder = []

        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "changelog" or edge._type == "~changelog":
                    changelog.append(edge)
                elif edge._type == "label" or edge._type == "~label":
                    label.append(edge)
                elif edge._type == "genericAttribute" or edge._type == "~genericAttribute":
                    genericAttribute.append(edge)
                elif edge._type == "measure" or edge._type == "~measure":
                    measure.append(edge)
                elif edge._type == "sharedWith" or edge._type == "~sharedWith":
                    sharedWith.append(edge)
                elif edge._type == "file" or edge._type == "~file":
                    file.append(edge)
                elif edge._type == "includes" or edge._type == "~includes":
                    includes.append(edge)
                elif edge._type == "thumbnail" or edge._type == "~thumbnail":
                    thumbnail.append(edge)
                elif edge._type == "folder" or edge._type == "~folder":
                    folder.append(edge)

        res = cls(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                  deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                  version=version, id=id, importJson=importJson, bitrate=bitrate, duration=duration,
                  endTime=endTime, fileLocation=fileLocation, startTime=startTime, caption=caption, exifData=exifData,
                  name=name, height=height, width=width, channels=channels, changelog=changelog, label=label,
                  genericAttribute=genericAttribute, measure=measure, sharedWith=sharedWith, file=file,
                  includes=includes, thumbnail=thumbnail, folder=folder)
        for e in res.get_all_edges(): e.source = res
        return res


class Video(Item):
    def __init__(self, dateAccessed=None, dateCreated=None, dateModified=None, deleted=None,
                 externalId=None, itemDescription=None, starred=None, version=None, id=None, importJson=None,
                 bitrate=None, duration=None, endTime=None, fileLocation=None, startTime=None, caption=None,
                 exifData=None, name=None, changelog=None, label=None, genericAttribute=None, measure=None,
                 sharedWith=None, file=None, includes=None, thumbnail=None, folder=None):
        super().__init__(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                         deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                         version=version, id=id, importJson=importJson, changelog=changelog, label=label,
                         genericAttribute=genericAttribute, measure=measure, sharedWith=sharedWith)
        self.bitrate = bitrate
        self.duration = duration
        self.endTime = endTime
        self.fileLocation = fileLocation
        self.startTime = startTime
        self.caption = caption
        self.exifData = exifData
        self.name = name
        self.file = file if file is not None else []
        self.includes = includes if includes is not None else []
        self.thumbnail = thumbnail if thumbnail is not None else []
        self.folder = folder if folder is not None else []

    @classmethod
    def from_json(cls, json):
        all_edges = json.get("allEdges", None)
        dateAccessed = json.get("dateAccessed", None)
        dateCreated = json.get("dateCreated", None)
        dateModified = json.get("dateModified", None)
        deleted = json.get("deleted", None)
        externalId = json.get("externalId", None)
        itemDescription = json.get("itemDescription", None)
        starred = json.get("starred", None)
        version = json.get("version", None)
        id = json.get("id", None)
        importJson = json.get("importJson", None)
        bitrate = json.get("bitrate", None)
        duration = json.get("duration", None)
        endTime = json.get("endTime", None)
        fileLocation = json.get("fileLocation", None)
        startTime = json.get("startTime", None)
        caption = json.get("caption", None)
        exifData = json.get("exifData", None)
        name = json.get("name", None)

        changelog = []
        label = []
        genericAttribute = []
        measure = []
        sharedWith = []
        file = []
        includes = []
        thumbnail = []
        folder = []

        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "changelog" or edge._type == "~changelog":
                    changelog.append(edge)
                elif edge._type == "label" or edge._type == "~label":
                    label.append(edge)
                elif edge._type == "genericAttribute" or edge._type == "~genericAttribute":
                    genericAttribute.append(edge)
                elif edge._type == "measure" or edge._type == "~measure":
                    measure.append(edge)
                elif edge._type == "sharedWith" or edge._type == "~sharedWith":
                    sharedWith.append(edge)
                elif edge._type == "file" or edge._type == "~file":
                    file.append(edge)
                elif edge._type == "includes" or edge._type == "~includes":
                    includes.append(edge)
                elif edge._type == "thumbnail" or edge._type == "~thumbnail":
                    thumbnail.append(edge)
                elif edge._type == "folder" or edge._type == "~folder":
                    folder.append(edge)

        res = cls(dateAccessed=dateAccessed, dateCreated=dateCreated, dateModified=dateModified,
                  deleted=deleted, externalId=externalId, itemDescription=itemDescription, starred=starred,
                  version=version, id=id, importJson=importJson, bitrate=bitrate, duration=duration,
                  endTime=endTime, fileLocation=fileLocation, startTime=startTime, caption=caption, exifData=exifData,
                  name=name, changelog=changelog, label=label, genericAttribute=genericAttribute, measure=measure,
                  sharedWith=sharedWith, file=file, includes=includes, thumbnail=thumbnail, folder=folder)
        for e in res.get_all_edges(): e.source = res
        return res

class Folder(Item):

    def __init__(self, name, path, folder=None, id=None, deleted=None):
        super().__init__(id=id, deleted=deleted)
        self.name = name
        self.path = path
        self.folder = folder if folder is not None else []

    @classmethod
    def from_json(cls, json):
        id = json.get("id", None)
        deleted = json.get("deleted", None)
        name = json.get("name", None)
        path = json.get("path", None)
        all_edges = json.get("allEdges", None)

        folder = []

        if all_edges is not None:
            for edge_json in all_edges:
                edge = Edge.from_json(edge_json)
                if edge._type == "folder" or edge._type == "~folder":
                    folder.append(edge)


        res = cls(name=name, path=path, folder=folder, id=id, deleted=deleted)
        for e in res.get_all_edges(): e.source = res
        return res
