from dropbox_plugin.plugin_flow.plugin_flow import PluginFlow
from dropbox_plugin.service_api.dropbox_service import DropboxService
from dropbox_plugin.data.schema import Photo, Video, Folder, File

SERVICE_NAME = 'dropbox'
NUM_LOGIN_TRIES = 3
# in case this is an app that uses OAuth 
APP_KEY = 'gj15s7uivtezkms'
APP_SECRET = 'j19aofn7obg8iqx'


class DropboxPlugin(PluginFlow):

    def __init__(self, client, plugin_id=None, run_id=None):
        super().__init__(client=client, plugin_id=plugin_id, run_id=run_id)
        self._add_to_schema()
        self.service = DropboxService(client)
        # activate plugin flow
        if plugin_id and run_id:
            # init login process
            self.start_auth_oauth()

    def run(self):
        if self.service.dbx is None:
            if self.plugin_id and self.run_id:
                self.start_auth_oauth()
        self.service.save_folder_content('')

    def start_auth_oauth(self):

        oauth_url = self.service.get_oauth_url(APP_KEY, APP_SECRET)
        for i in range(NUM_LOGIN_TRIES):
            account = self.ask_user_for_accounts(SERVICE_NAME, 'oauth-view', oauth_url)
            result = self.service.get_oauth_token(account.code)
            if result['success']:
                # set new token
                self.set_account_vars({
                    'refreshToken': result['refreshToken'],
                    'secret': result['token']
                })
                return True
            # set error message for UI
            self.set_account_vars({'errorMessage': 'Incorrect credentials'}, SERVICE_NAME)
        return False

    def _add_to_schema(self):
        photo = Photo(fileLocation='', exifData='', name='', height=1, width=1, channels=1)
        video = Video(bitrate=1, duration=1, fileLocation='', name='')
        folder = Folder(name='', path='')
        file = File(sha256='', fileLocation='', filename='', dateAccessed='')
        self.client.add_to_schema(photo)
        self.client.add_to_schema(file)
        self.client.add_to_schema(video)
        self.client.add_to_schema(folder)
