from pymemri.data.schema import CVUStoredDefinition
from pymemri.pod.client import PodClient

from dropbox_plugin.dropbox_plugin import DropboxPlugin

import sys, logging
from time import sleep

# SETUP
UI_POLLING_INTERVAL = 1
client = PodClient(url=sys.argv[1], database_key="a"*64, owner_key="plugin_test")
pf = DropboxPlugin(client)

# INSTALL
cvus = [CVUStoredDefinition(name="username-password-view"), CVUStoredDefinition(name="2fa-view"), CVUStoredDefinition(name='oauth-view')]
plugin_id = pf.install_plugin(name="Plugin Template", container=sys.argv[2], views=cvus)


# pf.set_account("username","pass", "test-service")
# RUN

# Dummy UI Client Simulation
def run_plugin(plugin_flow, interval=None):
    
    service_name = 'dropbox' # or None if it is only one account is used
    plugin_flow.trigger_plugin(interval=interval)

    # START POLLING
    while True:
        # take a breath
        sleep(UI_POLLING_INTERVAL)
        run = plugin_flow.get_run()
        plugin_flow.start()
        if plugin_flow.is_daemon():
            logging.warning("Started plugin in daemon mode.")
            return True
        
        if plugin_flow.is_completed():
            logging.warning("Completed plugin run. Well done.")
            return True

        if plugin_flow.is_user_action_needed():
    
            ##==============================
            ## THIS IS THE UI PART
            # Gets the active account
            # Gets the active View (CVU)
            # Displays the view to the user
            # Gets user input
            # Sets / Modifies the account
            # Changes the run state

            ## ==== BEGIN USER INTERFACE ====

            # get active account
            account = plugin_flow.get_account_from_plugin(service=service_name)

            ## DISPLAYS CVU & GETS USER INPUT

            view = plugin_flow.get_CVU(run)

            # This error message is displayed inside UI if exists
            if account and account.errorMessage: print(account.errorMessage)

            # User input - normally via CVU (view)
            username = input("Username: ")
            password = input("Password: ")
            oauth_code = input("OAuth Code: ")
            sleep(3)

            # Set account
            plugin_flow.set_account_vars({
                'service': service_name,
                'identifier': username,
                'secret': password, 
                'code': oauth_code,
                'errorMessage': None # reset error on each input
            }, service=service_name)  

            ## ==== END USER INTERFACE ====

            # Notify the plugin, continue the auth flow
            plugin_flow.complete_user_action()

# start plugin
run_plugin(pf)
