from dropbox_plugin.dropbox_plugin import DropboxPlugin
from pymemri.pod.client import PodClient

import os, json    

def main(pod_url=None, database_key=None, owner_key=None, plugin_item=None, start_plugin_id=None, auth_json=None):
    # setup client
    client = PodClient(url=pod_url, database_key=database_key, owner_key=owner_key, auth_json=auth_json)

    # ----- PLUGIN STARTS ------

    plugin = DropboxPlugin(client, plugin_id=plugin_item['id'], run_id=start_plugin_id)
    try:
        plugin.start()
    except Exception as e:
        plugin.failed(e)


    # ------ PLUGIN ENDS ------


def parse_environment_variables():
    plugin_data = {}
    env_variable_keys = {
        'auth_json': 'POD_AUTH_JSON',
        'owner_key': 'POD_OWNER',
        'plugin_item': 'POD_TARGET_ITEM',
        'pod_url': 'POD_FULL_ADDRESS',
        'start_plugin_id': 'POD_PLUGINRUN_ID'
    }
    # extract
    for k, v in env_variable_keys.items():
        if not v in os.environ:
            print(f"Missing environment variable: {v}. Exiting...")
            exit()
        plugin_data[k] = os.environ[v]
        # parse plugin and auth json
        if k in ['plugin_item', 'auth_json'] and plugin_data[k]:
            plugin_data[k] = json.loads(plugin_data[k])
    return plugin_data



# EXTRACT PARAMETERS FROM ENVIRONMENT
if __name__ == "__main__":

    plugin_data = parse_environment_variables()

    main(**plugin_data)
